@@ -23,16 +23,23 @@
 #include <string.h>
 #include <libintl.h>
 #include <hildon/hildon-program.h>
+#ifdef HILDON_HAS_APP_MENU
+#include <hildon/hildon-pannable-area.h>
+#endif
 #include <hildon/hildon-banner.h>
 #include <hildon/hildon-defines.h>
+#include <hildon-mime.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <glib/gprintf.h>
 #include <math.h>
+#include <tablet-browser-interface.h>
+#ifdef WITH_MODEST
+#include <libmodest-dbus-client/libmodest-dbus-client.h>
+#endif
 
-#include "callbacks.h"
 #include "metadata.h"
 #include "interface.h"
 #include "note.h"
@@ -41,13 +48,49 @@
 #include "app_data.h"
 #include "settings.h"
 #include "settings_window.h"
+#include "conboy_note_buffer.h"
+#include "ui_helper.h"
+#include "gregex.h"
 
-
+#include "callbacks.h"
 
 /* Private. TODO: Move to some public file */
 GtkTextTag* iter_get_depth_tag(GtkTextIter* iter);
 GtkTextTag* buffer_get_depth_tag(GtkTextBuffer *buffer, gint depth);
 
+gboolean __editing_title = FALSE;
+
+static void
+check_title(UserInterface *ui)
+{
+	if (__editing_title) {
+		__editing_title = FALSE;
+	} else {
+		return;
+	}
+
+	AppData *app_data = app_data_get();
+	ConboyNote *note = ui->note;
+	gchar *title = note_extract_title_from_buffer(ui->buffer);
+
+	ConboyNote *existing_note = conboy_note_store_find_by_title(app_data->note_store, title);
+	if (existing_note && (existing_note != note)) {
+		/* Display message */
+		gchar msg[1024];
+		g_sprintf(msg, _("<b>Note title taken</b>\n\nA note with the title <b>%s</b> already exists. Please choose another name for this note before continuing."), title);
+		ui_helper_show_confirmation_dialog(GTK_WINDOW(ui->window), msg, FALSE);
+
+		/* Select title */
+		GtkTextIter start, end;
+		gtk_text_buffer_get_start_iter(ui->buffer, &start);
+		end = start;
+		gtk_text_iter_forward_to_line_end(&end);
+		gtk_text_buffer_select_range(ui->buffer, &start, &end);
+	}
+
+	g_free(title);
+}
+
 static
 GtkTextTag* get_depth_tag_at_line(GtkTextBuffer *buffer, gint line_number)
 {
@@ -56,12 +99,12 @@ GtkTextTag* get_depth_tag_at_line(GtkTextBuffer *buffer, gint line_number)
 	return iter_get_depth_tag(&iter);
 }
 
-static void change_format(Note *note, GtkToggleAction *action)
+static void change_format(UserInterface *ui, GtkToggleAction *action)
 {
 	gint start_line, end_line, i;
 	GtkTextIter selection_start_iter, selection_end_iter;
 	GtkTextIter start_iter, end_iter;
-	GtkTextBuffer *buffer = GTK_TEXT_BUFFER(note->ui->buffer);
+	GtkTextBuffer *buffer = GTK_TEXT_BUFFER(ui->buffer);
 
 	const gchar *tag_name = gtk_action_get_name(GTK_ACTION(action));
 
@@ -107,7 +150,7 @@ static void change_format(Note *note, GtkToggleAction *action)
 		} else {
 			/* Nothing is selected, so this style should start from here on */
 			GtkTextTag *tag = gtk_text_tag_table_lookup(buffer->tag_table, tag_name);
-			note_add_active_tag(note, tag);
+			conboy_note_buffer_add_active_tag(CONBOY_NOTE_BUFFER(ui->buffer), tag);
 		}
 	} else {
 		/* The button just became deactive, so we should remove the formatting */
@@ -120,7 +163,7 @@ static void change_format(Note *note, GtkToggleAction *action)
 		} else {
 			/* Nothing is selected, so this style should stop from here on */
 			GtkTextTag *tag = gtk_text_tag_table_lookup(buffer->tag_table, tag_name);
-			note_remove_active_tag(note, tag);
+			conboy_note_buffer_remove_active_tag(CONBOY_NOTE_BUFFER(ui->buffer), tag);
 		}
 	}
 
@@ -131,8 +174,8 @@ void
 on_format_button_clicked               (GtkAction       *action,
 										gpointer		 user_data)
 {
-	Note *note = (Note*)user_data;
-	change_format(note, GTK_TOGGLE_ACTION(action));
+	UserInterface *ui = (UserInterface*)user_data;
+	change_format(ui, GTK_TOGGLE_ACTION(action));
 }
 
 void
@@ -140,8 +183,8 @@ on_font_size_radio_group_changed       (GtkRadioAction  *action,
 										GtkRadioAction  *current,
 		                                gpointer         user_data)
 {
-	Note *note = (Note*)user_data;
-	GtkTextBuffer *buffer = note->ui->buffer;
+	UserInterface *ui = (UserInterface*)user_data;
+	GtkTextBuffer *buffer = ui->buffer;
 	GtkTextIter start_iter;
 	GtkTextIter end_iter;
 	const gchar *tag_name = gtk_action_get_name(GTK_ACTION(current));
@@ -162,41 +205,61 @@ on_font_size_radio_group_changed       (GtkRadioAction  *action,
 
 	} else {
 		/* Remove all possible size tags */
-		note_remove_active_tag_by_name(note, "size:small");
-		note_remove_active_tag_by_name(note, "size:large");
-		note_remove_active_tag_by_name(note, "size:huge");
+		conboy_note_buffer_remove_active_tag_by_name(CONBOY_NOTE_BUFFER(ui->buffer), "size:small");
+		conboy_note_buffer_remove_active_tag_by_name(CONBOY_NOTE_BUFFER(ui->buffer), "size:large");
+		conboy_note_buffer_remove_active_tag_by_name(CONBOY_NOTE_BUFFER(ui->buffer), "size:huge");
 
 		/* If not normal size, add one size tag */
 		if (strcmp(tag_name, "size:normal") != 0) {
-			note_add_active_tag_by_name(note, tag_name);
+			conboy_note_buffer_add_active_tag_by_name(CONBOY_NOTE_BUFFER(ui->buffer), tag_name);
 		}
 	}
 }
 
-
 gboolean
 on_window_delete		               (GtkObject		*window,
 										GdkEvent		*event,
 										gpointer		 user_data)
 {
-	Note *note = (Note*)user_data;
-	g_printerr("Window delete \n");
-	note_save(note);
-	note_close_window(note);
+	UserInterface *ui = (UserInterface*)user_data;
+	GtkAdjustment *adj;
+
+	/* Save the current note */
+	note_save(ui);
+
+#ifdef HILDON_HAS_APP_MENU
+	adj = hildon_pannable_area_get_vadjustment(HILDON_PANNABLE_AREA(ui->scrolled_window));
+	/* Take screenshot for faster startup trick */
+	/* TODO: Maybe enable later again, now it causes confusion with scrolling / flickering */
+	/*hildon_gtk_window_take_screenshot(ui->window, TRUE);*/
+#else
+	adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(ui->scrolled_window));
+#endif
+
+	/* Save last viewd note and the scroll position */
+	settings_save_last_scroll_position(gtk_adjustment_get_value(adj));
+	settings_save_last_open_note(ui->note->guid);
+
+	/* Quit */
+	gtk_main_quit();
+
 	return TRUE; /* True to stop other handler from being invoked */
 }
 
 void on_quit_button_clicked(GtkAction *action, gpointer user_data)
 {
-	Note *note;
-	GList *open_notes = app_data_get()->open_notes;
+	UserInterface *ui = (UserInterface*)user_data;
+	/*
+	GList *open_windows = app_data_get()->open_windows;
 
-	while (open_notes != NULL) {
-		note = (Note*)open_notes->data;
-		note_save(note);
-		note_close_window(note);
-		open_notes = g_list_next(open_notes);
+	while (open_windows != NULL) {
+		ui = (UserInterface*)open_windows->data;
+		note_save(ui);
+		note_close_window(ui);
+		open_windows = g_list_next(open_windows);
 	}
+	*/
+	note_save(ui);
 
 	gtk_main_quit();
 }
@@ -211,8 +274,20 @@ void
 on_new_button_clicked					(GtkAction		*action,
 										 gpointer		 user_data)
 {
-	Note *note = note_create_new();
-	note_show(note);
+	AppData *app_data = app_data_get();
+	UserInterface *ui = (UserInterface*)user_data;
+
+	/* Save current note */
+	note_save(ui);
+
+	/* Create new note */
+	int num = conboy_note_store_get_length(app_data->note_store) + 1;
+	gchar title[100];
+
+	g_sprintf(title, _("New Note %i"), num);
+
+	ConboyNote *note = conboy_note_new_with_title(title);
+	note_show(note, TRUE, TRUE, TRUE);
 }
 
 static void
@@ -311,54 +386,65 @@ remove_bullets(GtkTextBuffer *buffer, GtkTextIter *start_iter, GtkTextIter *end_
 	}
 }
 
+static void
+enable_bullets(GtkTextBuffer *buffer)
+{
+	GtkTextIter start_iter, end_iter;
+
+	if (gtk_text_buffer_get_has_selection(buffer)) {
+		/* Something is selected */
+		gtk_text_buffer_get_selection_bounds(buffer, &start_iter, &end_iter);
+		add_bullets(buffer, gtk_text_iter_get_line(&start_iter), gtk_text_iter_get_line(&end_iter), buffer_get_depth_tag(buffer, 1));
+
+	} else {
+		/* Nothing is selected */
+		int line;
+		gtk_text_buffer_get_iter_at_mark(buffer, &start_iter, gtk_text_buffer_get_insert(buffer));
+		line = gtk_text_iter_get_line(&start_iter);
+		add_bullets(buffer, line, line, buffer_get_depth_tag(buffer, 1));
+
+		conboy_note_buffer_add_active_tag_by_name(CONBOY_NOTE_BUFFER(buffer), "list-item");
+		conboy_note_buffer_add_active_tag_by_name(CONBOY_NOTE_BUFFER(buffer), "list");
+	}
+}
+
+static void
+disable_bullets(GtkTextBuffer *buffer)
+{
+	GtkTextIter start_iter, end_iter;
+
+	if (gtk_text_buffer_get_has_selection(buffer)) {
+		/* Something is selected */
+		gtk_text_buffer_get_selection_bounds(buffer, &start_iter, &end_iter);
+		remove_bullets(buffer, &start_iter, &end_iter);
+
+		conboy_note_buffer_remove_active_tag_by_name(CONBOY_NOTE_BUFFER(buffer), "list-item");
+		conboy_note_buffer_remove_active_tag_by_name(CONBOY_NOTE_BUFFER(buffer), "list");
+
+	} else {
+		/* Nothing is selected */
+		gtk_text_buffer_get_iter_at_mark(buffer, &start_iter, gtk_text_buffer_get_insert(buffer));
+		gtk_text_buffer_get_iter_at_mark(buffer, &end_iter, gtk_text_buffer_get_insert(buffer));
+		remove_bullets(buffer, &start_iter, &end_iter);
+
+		conboy_note_buffer_remove_active_tag_by_name(CONBOY_NOTE_BUFFER(buffer), "list-item");
+		conboy_note_buffer_remove_active_tag_by_name(CONBOY_NOTE_BUFFER(buffer), "list");
+	}
+}
+
 void
 on_bullets_button_clicked				(GtkAction		*action,
 										 gpointer		 user_data)
 {
-	Note *note = (Note*)user_data;
-	UserInterface *ui = note->ui;
+	UserInterface *ui = (UserInterface*)user_data;
 	GtkTextBuffer *buffer = ui->buffer;
-	GtkTextIter start_iter, end_iter;
 
 	if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action))) {
 		/* The button just became active, so we should enable the formatting */
-		if (gtk_text_buffer_get_has_selection(buffer)) {
-			/* Something is selected */
-			gtk_text_buffer_get_selection_bounds(buffer, &start_iter, &end_iter);
-			add_bullets(buffer, gtk_text_iter_get_line(&start_iter), gtk_text_iter_get_line(&end_iter), buffer_get_depth_tag(buffer, 1));
-
-		} else {
-			/* Nothing is selected */
-			GtkTextTag *tag;
-			int line;
-			gtk_text_buffer_get_iter_at_mark(buffer, &start_iter, gtk_text_buffer_get_insert(buffer));
-			line = gtk_text_iter_get_line(&start_iter);
-			add_bullets(buffer, line, line, buffer_get_depth_tag(buffer, 1));
-
-			tag = gtk_text_tag_table_lookup(buffer->tag_table, "list-item");
-			note_add_active_tag(note, tag);
-			tag = gtk_text_tag_table_lookup(buffer->tag_table, "list");
-			note_add_active_tag(note, tag);
-		}
+		enable_bullets(buffer);
 	} else {
 		/* The button just became deactive, so we should remove the formatting */
-		if (gtk_text_buffer_get_has_selection(buffer)) {
-			/* Something is selected */
-			gtk_text_buffer_get_selection_bounds(buffer, &start_iter, &end_iter);
-			remove_bullets(buffer, &start_iter, &end_iter);
-
-			note_remove_active_tag_by_name(note, "list-item");
-			note_remove_active_tag_by_name(note, "list");
-
-		} else {
-			/* Nothing is selected */
-			gtk_text_buffer_get_iter_at_mark(buffer, &start_iter, gtk_text_buffer_get_insert(buffer));
-			gtk_text_buffer_get_iter_at_mark(buffer, &end_iter, gtk_text_buffer_get_insert(buffer));
-			remove_bullets(buffer, &start_iter, &end_iter);
-
-			note_remove_active_tag_by_name(note, "list-item");
-			note_remove_active_tag_by_name(note, "list");
-		}
+		disable_bullets(buffer);
 	}
 
 }
@@ -367,8 +453,8 @@ void
 on_link_button_clicked				   (GtkAction		*action,
 										gpointer		 user_data)
 {
-	Note *note = (Note*)user_data;
-	GtkTextBuffer *buffer = note->ui->buffer;
+	UserInterface *ui = (UserInterface*)user_data;
+	GtkTextBuffer *buffer = ui->buffer;
 	GtkTextIter start, end;
 	const gchar *text;
 
@@ -379,8 +465,10 @@ on_link_button_clicked				   (GtkAction		*action,
 	gtk_text_buffer_get_selection_bounds(buffer, &start, &end);
 	text = gtk_text_iter_get_text(&start, &end);
 	gtk_text_buffer_apply_tag_by_name(buffer, "link:internal", &start, &end);
+
+	/* Make it save */
 	gtk_text_buffer_set_modified(buffer, TRUE);
-	
+
 	note_show_by_title(text);
 }
 
@@ -388,85 +476,27 @@ void
 on_notes_button_clicked				   (GtkAction		*action,
 										gpointer		 user_data) {
 
-	AppData *app_data;
-	GTimer *timer;
-	gulong micro;
-
-	app_data = app_data_get();
-	if (app_data->note_store == NULL) {
-		return;
-	}
-
-	timer = g_timer_new();
+	AppData *app_data = app_data_get();
+	g_return_if_fail(app_data->note_store != NULL);
 
 	search_window_open();
-
-	g_timer_stop(timer);
-	g_timer_elapsed(timer, &micro);
-	g_timer_destroy(timer);
-	g_printerr("Opening search window: %lu micro seconds \n", micro);
-
 }
 
-/* TODO: The signal "mark-set" is emitted 4 times when clicking into the text. While selecting
- * it's emitted continuesly. */
-void
-on_textview_cursor_moved			   (GtkTextBuffer	*buffer,
-										GtkTextIter		*location,
-										GtkTextMark		*mark,
-										gpointer		 user_data)
+static void
+update_button_states(UserInterface *ui)
 {
-	Note *note = (Note*)user_data;
-	UserInterface *ui = note->ui;
-	GSList *tags;
-	GtkTextTag *tag;
-	const gchar *mark_name;
-
-	/* Only enable the link action, if something is selected */
-	gtk_action_set_sensitive(ui->action_link, gtk_text_buffer_get_has_selection(buffer));
-
-	/* We only do something if the "insert" mark changed. */
-	mark_name = gtk_text_mark_get_name(mark);
-	if ((mark_name == NULL) || (strcmp(mark_name, "insert") != 0)) {
-		return;
-	}
-
-	/* Clean the list of active tags */
-	g_slist_free(note->active_tags);
-
-	/* Add tags at this location */
-	note->active_tags = gtk_text_iter_get_tags(location);
-
-	/* Go the beginning of line and check if there is a bullet.
-	 * If yes, add list-item and list tags and enable the indent actions */
-	gtk_text_iter_set_line_offset(location, 0);
-	if (iter_get_depth_tag(location) != NULL) {
-		/* Add tags */
-		note_add_active_tag_by_name(note, "list-item");
-		note_add_active_tag_by_name(note, "list");
-		/* Enable indent actions */
-		gtk_action_set_sensitive(ui->action_inc_indent, TRUE);
-		gtk_action_set_sensitive(ui->action_dec_indent, TRUE);
-	} else {
-		/* Disable indent actions */
-		gtk_action_set_sensitive(ui->action_inc_indent, FALSE);
-		gtk_action_set_sensitive(ui->action_dec_indent, FALSE);
-	}
-
-	/* Copy pointer for iteration */
-	tags = note->active_tags;
-
-
 	/* Blocking signals here because the ..set_active() method makes the buttons
 	 * emit the clicked signal. And because of this the formatting changes.
 	 */
-	g_signal_handlers_block_by_func(ui->action_bold, on_format_button_clicked, note);
-	g_signal_handlers_block_by_func(ui->action_italic, on_format_button_clicked, note);
-	g_signal_handlers_block_by_func(ui->action_strike, on_format_button_clicked, note);
-	g_signal_handlers_block_by_func(ui->action_highlight, on_format_button_clicked, note);
-	g_signal_handlers_block_by_func(ui->action_fixed, on_format_button_clicked, note);
-	g_signal_handlers_block_by_func(ui->action_bullets, on_bullets_button_clicked, note);
-	g_signal_handlers_block_by_func(ui->action_font_small, on_font_size_radio_group_changed, note);
+	g_signal_handlers_block_by_func(ui->action_bold, on_format_button_clicked, ui);
+	g_signal_handlers_block_by_func(ui->action_italic, on_format_button_clicked, ui);
+	g_signal_handlers_block_by_func(ui->action_strike, on_format_button_clicked, ui);
+	g_signal_handlers_block_by_func(ui->action_highlight, on_format_button_clicked, ui);
+	g_signal_handlers_block_by_func(ui->action_fixed, on_format_button_clicked, ui);
+	g_signal_handlers_block_by_func(ui->action_bullets, on_bullets_button_clicked, ui);
+	g_signal_handlers_block_by_func(ui->action_font_small, on_font_size_radio_group_changed, ui);
+
+
 
 	/* TODO: This can be optimized: Note disable all and then enable selected, but determine state and then
 	 * set the state. */
@@ -478,8 +508,14 @@ on_textview_cursor_moved			   (GtkTextBuffer	*buffer,
 	gtk_toggle_action_set_active(ui->action_bullets, FALSE);
 	gtk_radio_action_set_current_value(ui->action_font_small, 1); /* Enable normal font size */
 
+	/* Disable indent actions */
+	/*gtk_action_set_sensitive(ui->action_inc_indent, TRUE);*/
+	gtk_action_set_sensitive(ui->action_dec_indent, FALSE);
+
+	/* Copy pointer for iteration */
+	GSList *tags = conboy_note_buffer_get_active_tags(CONBOY_NOTE_BUFFER(ui->buffer));
 	while (tags != NULL) {
-		tag = GTK_TEXT_TAG(tags->data);
+		GtkTextTag *tag = GTK_TEXT_TAG(tags->data);
 		if (strcmp(tag->name, "bold") == 0) {
 			gtk_toggle_action_set_active(ui->action_bold, TRUE);
 		} else if (strcmp(tag->name, "italic") == 0) {
@@ -498,19 +534,70 @@ on_textview_cursor_moved			   (GtkTextBuffer	*buffer,
 			gtk_radio_action_set_current_value(ui->action_font_small, 2);
 		} else if (strcmp(tag->name, "size:huge") == 0) {
 			gtk_radio_action_set_current_value(ui->action_font_small, 3);
+		} else if (strcmp(tag->name, "list") == 0) {
+			/*gtk_action_set_sensitive(ui->action_inc_indent, TRUE);*/
+			gtk_action_set_sensitive(ui->action_dec_indent, TRUE);
 		}
 
 		tags = tags->next;
 	}
 
 	/* unblock signals */
-	g_signal_handlers_unblock_by_func(ui->action_bold, on_format_button_clicked, note);
-	g_signal_handlers_unblock_by_func(ui->action_italic, on_format_button_clicked, note);
-	g_signal_handlers_unblock_by_func(ui->action_strike, on_format_button_clicked, note);
-	g_signal_handlers_unblock_by_func(ui->action_highlight, on_format_button_clicked, note);
-	g_signal_handlers_unblock_by_func(ui->action_fixed, on_format_button_clicked, note);
-	g_signal_handlers_unblock_by_func(ui->action_bullets, on_bullets_button_clicked, note);
-	g_signal_handlers_unblock_by_func(ui->action_font_small, on_font_size_radio_group_changed, note);
+	g_signal_handlers_unblock_by_func(ui->action_bold, on_format_button_clicked, ui);
+	g_signal_handlers_unblock_by_func(ui->action_italic, on_format_button_clicked, ui);
+	g_signal_handlers_unblock_by_func(ui->action_strike, on_format_button_clicked, ui);
+	g_signal_handlers_unblock_by_func(ui->action_highlight, on_format_button_clicked, ui);
+	g_signal_handlers_unblock_by_func(ui->action_fixed, on_format_button_clicked, ui);
+	g_signal_handlers_unblock_by_func(ui->action_bullets, on_bullets_button_clicked, ui);
+	g_signal_handlers_unblock_by_func(ui->action_font_small, on_font_size_radio_group_changed, ui);
+}
+
+static void
+update_active_tags(GtkTextBuffer *buffer, GtkTextIter *location)
+{
+	/* Clean the list of active tags */
+	conboy_note_buffer_clear_active_tags(CONBOY_NOTE_BUFFER(buffer));
+
+	/* Add tags at this location */
+	GSList *tags = gtk_text_iter_get_tags(location);
+	if (tags != NULL) {
+		conboy_note_buffer_set_active_tags(CONBOY_NOTE_BUFFER(buffer), tags);
+	}
+
+	/* Go the beginning of line and check if there is a bullet.
+	 * If yes, add list-item and list tags and enable the indent actions */
+	gtk_text_iter_set_line_offset(location, 0);
+	if (iter_get_depth_tag(location) != NULL) {
+		/* Add tags */
+		conboy_note_buffer_add_active_tag_by_name(CONBOY_NOTE_BUFFER(buffer), "list-item");
+		conboy_note_buffer_add_active_tag_by_name(CONBOY_NOTE_BUFFER(buffer), "list");
+	}
+}
+
+/* TODO: The signal "mark-set" is emitted 4 times when clicking into the text. While selecting
+ * it's emitted continuesly. */
+void
+on_textview_cursor_moved			   (GtkTextBuffer	*buffer,
+										GtkTextIter		*location,
+										GtkTextMark		*mark,
+										gpointer		 user_data)
+{
+	UserInterface *ui = (UserInterface*)user_data;
+	const gchar *mark_name;
+
+	/* Only enable the link action, if something is selected */
+	gtk_action_set_sensitive(ui->action_link, gtk_text_buffer_get_has_selection(buffer));
+
+	/* We only do something if the "insert" mark changed. */
+	mark_name = gtk_text_mark_get_name(mark);
+	if ((mark_name == NULL) || (strcmp(mark_name, "insert") != 0)) {
+		return;
+	}
+
+	check_title(ui);
+
+	update_active_tags(ui->buffer, location);
+	update_button_states(ui);
 }
 
 void
@@ -578,11 +665,140 @@ on_link_internal_tag_event				(GtkTextTag  *tag,
 	return FALSE;
 }
 
+static void
+open_url(gchar *url)
+{
+	AppData *app_data = app_data_get();
+
+	g_printerr("INFO: Open URL: >%s<\n", url);
+
+	DBusConnection *con = osso_get_dbus_connection(app_data->osso_ctx);
+
+	if (hildon_mime_open_file(con, url)) {
+		/* Everything ok. File was opened */
+		g_printerr("INFO: URL successfull opened\n");
+		return;
+	}
+
+	g_printerr("INFO: Cannot open URL with hildon_mime. Falling back to hardcoded values\n");
+
+	if (strncmp(url, "http", 4) == 0 || strncmp(url, "ftp", 3) == 0)
+	{
+		g_printerr("Trying to open in browser\n");
+		/* Open in browser */
+		osso_rpc_run_with_defaults(app_data->osso_ctx, "osso_browser",
+				OSSO_BROWSER_OPEN_NEW_WINDOW_REQ, NULL,
+				DBUS_TYPE_STRING, url, DBUS_TYPE_INVALID);
+		return;
+	}
+
+	if (strncmp(url, "mailto", 6) == 0)
+	{
+		/* Open in Modest */
+		#ifdef WITH_MODEST
+		g_printerr("Trying to open with modest\n");
+		libmodest_dbus_client_mail_to(app_data->osso_ctx, url);
+		#endif
+		return;
+	}
+
+	if (strncmp(url, "file", 4) == 0)
+	{
+		/* Open in Filemanager */
+		g_printerr("Trying to open in file manager \n");
+
+		/* Test if is a path and the path exits */
+		gchar *path = url + 7;
+		g_printerr("Path: %s\n", path);
+
+		if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
+			osso_rpc_run_with_defaults(app_data->osso_ctx, "osso_filemanager",
+					"open_folder", NULL,
+					DBUS_TYPE_STRING, path, DBUS_TYPE_INVALID);
+			return;
+		}
+	}
+
+	hildon_banner_show_informationf(GTK_WIDGET(app_data->note_window->window),
+			NULL, "Cannot open '%s'", url);
+}
+
+/**
+ * Create a proper URL out of the given link.
+ *
+ * Returns: Newly allocated string
+ */
+static gchar*
+create_url(gchar *link)
+{
+	gchar *result;
+
+	if (strncmp(link, "www.", 4) == 0) {
+		result = g_strconcat("http://", link, NULL);
+	}
+	else if (strncmp(link, "/", 1) == 0 && strncmp(link, "//", 2) != 0) {
+		result = g_strconcat("file://", link, NULL);
+	}
+	else if (strncmp(link, "~/", 2) == 0) {
+		gchar *path = link + 1; /* Cut of the tilde char */
+		gchar *file = g_build_filename(g_get_home_dir(), path, NULL);
+		result = g_strconcat("file://", file, NULL);
+		g_free(file);
+	}
+	else if (g_regex_match_simple("^(?!(news|mailto|http|https|ftp|file|irc):).+@.{2,}$", link, G_REGEX_CASELESS, 0)) {
+		result = g_strconcat("mailto:", link, NULL);
+	}
+	else {
+		result = g_strdup(link);
+	}
+
+	return result;
+}
+
+gboolean
+on_link_url_tag_event				(GtkTextTag  *tag,
+										 GObject     *object,
+										 GdkEvent    *event,
+										 GtkTextIter *iter,
+										 gpointer     user_data)
+{
+	GdkEventType type = event->type;
+	GtkTextIter start;
+	gchar *link_text;
+
+	if (type == GDK_BUTTON_RELEASE) {
+		if (((GdkEventButton*)event)->button == 1) {
+
+			if (!gtk_text_iter_begins_tag(iter, tag)) {
+				gtk_text_iter_backward_to_tag_toggle(iter, tag);
+			}
+
+			start = *iter;
+
+			if (!gtk_text_iter_ends_tag(iter, tag)) {
+				gtk_text_iter_forward_to_tag_toggle(iter, tag);
+			}
+
+			link_text = gtk_text_iter_get_text(&start, iter);
+
+			g_printerr("Link: >%s< \n", link_text);
+
+			gchar *url = create_url(link_text);
+			g_free(link_text);
+
+			open_url(url);
+		}
+	}
+
+	return FALSE;
+}
+
 void
 on_delete_button_clicked			   (GtkAction		*action,
 										gpointer		 user_data)
 {
-	Note *note = (Note*)user_data;
+	UserInterface *ui = (UserInterface*)user_data;
+	AppData *app_data = app_data_get();
 
 	gchar *message = g_strconcat("<b>",
 			_("Really delete this note?"),
@@ -590,33 +806,39 @@ on_delete_button_clicked			   (GtkAction		*action,
 			_("If you delete a note it is permanently lost."),
 			NULL);
 
-	GtkWidget *dialog = gtk_message_dialog_new_with_markup(
-			GTK_WINDOW(note->ui->window),
-			GTK_DIALOG_MODAL,
-			GTK_MESSAGE_QUESTION,
-			GTK_BUTTONS_YES_NO,
-			message);
-
+	GtkWidget *dialog = ui_helper_create_yes_no_dialog(GTK_WINDOW(ui->window), message);
 	gint response = gtk_dialog_run(GTK_DIALOG(dialog));
-	
 	gtk_widget_destroy(dialog);
+	g_free(message);
 
 	if (response == GTK_RESPONSE_YES) {
-		/* Delete note and close window */
-		note_delete(note);
-		note_close_window(note);
-		note_free(note);
-	}
 
-	g_free(message);
+		/* Delete note */
+		note_delete(ui->note);
+		g_object_unref(ui->note);
+		ui->note = NULL;
+
+		/* Show previous note */
+		ConboyNote *note;
+		if (app_data->current_element == NULL) {
+			note = conboy_note_store_get_latest(app_data->note_store);
+			app_data->note_history = g_list_append(app_data->note_history, note);
+			app_data->current_element = app_data->note_history;
+			note_show(note, FALSE, TRUE, FALSE);
+		} else {
+			note = CONBOY_NOTE(app_data->current_element->data);
+			note_show(note, FALSE, TRUE, FALSE);
+		}
+
+	}
 }
 
 static
-gboolean note_save_callback(Note *note)
+gboolean note_save_callback(UserInterface *ui)
 {
 	/* Test, to see if the window is still open */
-	if (note != NULL && GTK_IS_TEXT_BUFFER(note->ui->buffer)) {
-		note_save(note);
+	if (ui->note != NULL && GTK_IS_TEXT_BUFFER(ui->buffer)) {
+		note_save(ui);
 	}
 
 	/* Return FALSE to make the Timer stop */
@@ -627,28 +849,27 @@ void
 on_text_buffer_modified_changed			(GtkTextBuffer *buffer,
 										 gpointer		user_data)
 {
-	Note *note = (Note*)user_data;
+	UserInterface *ui = (UserInterface*)user_data;
 
-	if (!gtk_text_buffer_get_modified(buffer)) {
-		/*g_printerr("Buffer changed from dirty to saved \n");*/
+	if (!gtk_text_buffer_get_modified(ui->buffer)) {
 		return;
 	}
 
-	/*g_printerr("Buffer is dirty. Saving in 10 seconds\n");*/
+	g_printerr("Buffer is dirty\n");
 
-	/* Save 10 seconds after the buffer got dirty */
-	g_timeout_add(4000, (GSourceFunc)note_save_callback, note);
+	/* Save 4 seconds after the buffer got dirty */
+	g_timeout_add(4000, (GSourceFunc)note_save_callback, ui);
 }
 
 static
 void change_font_size_by(gint change)
 {
 	gint size;
-	
+
 	if (change == 0) {
 		return;
 	}
-	
+
 	size = settings_load_font_size();
 
 	if (change > 0) {
@@ -664,7 +885,7 @@ void change_font_size_by(gint change)
 			return;
 		}
 	}
-	
+
 	settings_save_font_size(size);
 }
 
@@ -686,10 +907,9 @@ gboolean on_hardware_key_pressed	(GtkWidget			*widget,
 									 GdkEventKey		*event,
 									 gpointer           user_data)
 {
-	Note *note = (Note*)user_data;
-	GtkWidget *window = GTK_WIDGET(note->ui->window);
+	UserInterface *ui = (UserInterface*)user_data;
+	GtkWidget *window = GTK_WIDGET(ui->window);
 	AppData *app_data = app_data_get();
-	GList *open_notes;
 
 	switch (event->keyval) {
 	case HILDON_HARDKEY_INCREASE:
@@ -703,35 +923,12 @@ gboolean on_hardware_key_pressed	(GtkWidget			*widget,
 		return TRUE;
 
 	case HILDON_HARDKEY_ESC:
-		on_window_delete(GTK_OBJECT(window), NULL, note);
+		on_window_delete(GTK_OBJECT(window), NULL, ui->note);
 		return TRUE;
 
 	case HILDON_HARDKEY_FULLSCREEN:
 		/* Toggle fullscreen */
-		app_data->fullscreen = !app_data->fullscreen;
-
-		/* Set all open windows to fullscreen or unfullscreen */
-		open_notes = app_data->open_notes;
-		while (open_notes != NULL) {
-			Note *open_note = (Note*)open_notes->data;
-			if (app_data->fullscreen) {
-				gtk_window_fullscreen(GTK_WINDOW(open_note->ui->window));
-			} else {
-				gtk_window_unfullscreen(GTK_WINDOW(open_note->ui->window));
-			}
-			open_notes = open_notes->next;
-		}
-		/* Set search window to fullscreen or unfullscreen */
-		if (app_data->search_window != NULL) {
-			if (app_data->fullscreen) {
-				gtk_window_fullscreen(GTK_WINDOW(app_data->search_window));
-			} else {
-				gtk_window_unfullscreen(GTK_WINDOW(app_data->search_window));
-			}
-		}
-
-		/* Focus again on the active note */
-		note_set_focus(note);
+		ui_helper_toggle_fullscreen(GTK_WINDOW(ui->window));
 		return TRUE;
 	}
 
@@ -783,10 +980,10 @@ gboolean line_needs_bullet(GtkTextBuffer *buffer, gint line_number)
 }
 
 /* TODO: This function must be cut into smaller parts */
-static gboolean add_new_line(Note *note)
+static gboolean add_new_line(UserInterface *ui)
 {
-	GtkTextBuffer *buffer = note->ui->buffer;
-	GtkTextView *view = note->ui->view;
+	GtkTextBuffer *buffer = ui->buffer;
+	GtkTextView *view = ui->view;
 	GtkTextIter iter;
 	GtkTextTag *depth_tag;
 	gint line;
@@ -804,10 +1001,9 @@ static gboolean add_new_line(Note *note)
 			GSList *tmp;
 
 			/* Remove all tags but <list> from active tags */
-			tmp = g_slist_copy(note->active_tags);
-			g_slist_free(note->active_tags);
-			note->active_tags = NULL;
-			note_add_active_tag_by_name(note, "list");
+			tmp = g_slist_copy(conboy_note_buffer_get_active_tags(CONBOY_NOTE_BUFFER(buffer)));
+			conboy_note_buffer_clear_active_tags(CONBOY_NOTE_BUFFER(buffer));
+			conboy_note_buffer_add_active_tag_by_name(CONBOY_NOTE_BUFFER(ui->buffer), "list");
 
 			/* Insert newline and bullet */
 			gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
@@ -817,20 +1013,17 @@ static gboolean add_new_line(Note *note)
 			add_bullets(buffer, line, line, depth_tag);
 
 			/* Add all tags back to active tags */
-			note->active_tags = tmp;
+			conboy_note_buffer_set_active_tags(CONBOY_NOTE_BUFFER(buffer), tmp);
 
 			return TRUE;
 
 		} else {
 			/* Remove bullet and insert newline */
-			GtkTextTag *tag;
 			GtkTextIter start = iter;
 
 			/* Disable list and list-item tags */
-			tag = gtk_text_tag_table_lookup(buffer->tag_table, "list-item");
-			note_remove_active_tag(note, tag);
-			tag = gtk_text_tag_table_lookup(buffer->tag_table, "list");
-			note_remove_active_tag(note, tag);
+			conboy_note_buffer_remove_active_tag_by_name(CONBOY_NOTE_BUFFER(ui->buffer), "list-item");
+			conboy_note_buffer_remove_active_tag_by_name(CONBOY_NOTE_BUFFER(ui->buffer), "list");
 
 			/* Delete the bullet and the last newline */
 			gtk_text_iter_set_line_offset(&start, 0);
@@ -841,7 +1034,7 @@ static gboolean add_new_line(Note *note)
 			gtk_text_buffer_insert(buffer, &iter, "\n", -1);
 
 			/* Disable the bullet button */
-			on_textview_cursor_moved(buffer, &iter, gtk_text_buffer_get_insert(buffer) ,note);
+			on_textview_cursor_moved(buffer, &iter, gtk_text_buffer_get_insert(buffer) ,ui);
 
 			return TRUE;
 		}
@@ -874,10 +1067,9 @@ static gboolean add_new_line(Note *note)
 			/* TODO: Copied from above */
 
 			/* Remove all tags but <list> from active tags */
-			tmp = g_slist_copy(note->active_tags);
-			g_slist_free(note->active_tags);
-			note->active_tags = NULL;
-			note_add_active_tag_by_name(note, "list");
+			tmp = g_slist_copy(conboy_note_buffer_get_active_tags(CONBOY_NOTE_BUFFER(ui->buffer)));
+			conboy_note_buffer_clear_active_tags(CONBOY_NOTE_BUFFER(ui->buffer));
+			conboy_note_buffer_add_active_tag_by_name(CONBOY_NOTE_BUFFER(ui->buffer), "list");
 
 			/* Insert newline and bullet */
 			gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
@@ -886,14 +1078,15 @@ static gboolean add_new_line(Note *note)
 			add_bullets(buffer, line, line, buffer_get_depth_tag(buffer, 1));
 
 			/* Add all tags back to active tags */
-			note->active_tags = tmp;
+			/*ui->active_tags = tmp;*/
+			conboy_note_buffer_set_active_tags(CONBOY_NOTE_BUFFER(ui->buffer), tmp);
 
 			/* Turn on the list-item tag from here on */
-			note_add_active_tag_by_name(note, "list-item");
+			conboy_note_buffer_add_active_tag_by_name(CONBOY_NOTE_BUFFER(ui->buffer), "list-item");
 
 			/* Revaluate (turn on) the bullet button */
 			gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
-			on_textview_cursor_moved(buffer, &iter, gtk_text_buffer_get_insert(buffer), note);
+			on_textview_cursor_moved(buffer, &iter, gtk_text_buffer_get_insert(buffer), ui);
 
 			return TRUE;
 		}
@@ -909,12 +1102,12 @@ on_text_view_key_pressed                      (GtkWidget   *widget,
                                                GdkEventKey *event,
                                                gpointer     user_data)
 {
-	Note *note = (Note*)user_data;
+	UserInterface *ui = (UserInterface*)user_data;
 
 	switch (event->keyval) {
 		case GDK_Return:
 		case GDK_KP_Enter:
-			return add_new_line(note);
+			return add_new_line(ui);
 		default:
 			return FALSE;
 	}
@@ -922,15 +1115,10 @@ on_text_view_key_pressed                      (GtkWidget   *widget,
 }
 
 static void
-apply_active_tags(GtkTextBuffer *buffer, GtkTextIter *iter, const gchar *input, Note *note)
+apply_active_tags(GtkTextBuffer *buffer, GtkTextIter *iter, const gchar *input, UserInterface *ui)
 {
 	GtkTextIter start_iter;
-	GSList *active_tags = note->active_tags;
-
-	/* Only apply active tags on typed text, not on pasted text */
-	if (g_utf8_strlen(input, -1) > 1) {
-		return;
-	}
+	GSList *active_tags = conboy_note_buffer_get_active_tags(CONBOY_NOTE_BUFFER(buffer));
 
 	/* The first line is always the title. Don't apply tags there */
 	if (gtk_text_iter_get_line(iter) == 0) {
@@ -956,17 +1144,20 @@ on_text_buffer_insert_text					(GtkTextBuffer *buffer,
 											 gint			len,
 											 gpointer		user_data)
 {
-	Note *note = (Note*)user_data;
+	UserInterface *ui = (UserInterface*)user_data;
 	GtkTextIter start_iter, end_iter;
 	GTimer *timer;
 	gulong micro;
 
 	/* Don't do anything when in the title line */
 	if (gtk_text_iter_get_line(iter) == 0) {
+		__editing_title = TRUE;
 		return;
 	}
 
-	apply_active_tags(buffer, iter, text, note);
+	check_title(ui);
+
+	apply_active_tags(buffer, iter, text, ui);
 
 	timer = g_timer_new();
 
@@ -975,8 +1166,10 @@ on_text_buffer_insert_text					(GtkTextBuffer *buffer,
 
 	/* Move start iter back to the position before the insert */
 	gtk_text_iter_backward_chars(&start_iter, g_utf8_strlen(text, -1));
-	
-	auto_highlight_links(note, &start_iter, &end_iter);
+
+	auto_highlight_links(ui, &start_iter, &end_iter);
+
+	auto_highlight_urls(ui, &start_iter, &end_iter);
 
 	g_timer_stop(timer);
 	g_timer_elapsed(timer, &micro);
@@ -990,38 +1183,19 @@ on_text_buffer_delete_range					(GtkTextBuffer *buffer,
 											 GtkTextIter   *end_iter,
 											 gpointer		user_data)
 {
-	Note *note = (Note*)user_data;
-	/*
-	GtkTextTag *link_tag;
-	gint max_len;
-	*/
+	UserInterface *ui = (UserInterface*)user_data;
 
 	/* Don't do anything when in the title line */
 	if (gtk_text_iter_get_line(start_iter) == 0 || gtk_text_iter_get_line(end_iter) == 0) {
+		__editing_title = TRUE;
 		return;
 	}
 
-	auto_highlight_links(note, start_iter, end_iter);
-	
-	/*
-	link_tag = gtk_text_tag_table_lookup(buffer->tag_table, "link:internal");
-	max_len = get_length_of_longest_title();
-	*/
+	check_title(ui);
 
-	/* This handler runs after the default handler, so we can modify the iters as we like */
-	/*
-	extend_block(start_iter, end_iter, max_len, link_tag);
-	*/
+	auto_highlight_links(ui, start_iter, end_iter);
 
-	/* Remove link tags */
-	/*
-	gtk_text_buffer_remove_tag(buffer, link_tag, start_iter, end_iter);
-	*/
-
-	/* Add link tags */
-	/*
-	highlight_titles(note, buffer, start_iter, end_iter);
-	*/
+	auto_highlight_urls(ui, start_iter, end_iter);
 }
 
 /*
@@ -1112,6 +1286,9 @@ void increase_indent(GtkTextBuffer *buffer, gint start_line, gint end_line)
 
 			/* Add new bullet with new tag */
 			add_bullets(buffer, i, i, new_tag);
+		} else {
+			/* Not yet a bullet list - create list */
+			enable_bullets(buffer);
 		}
 	}
 }
@@ -1132,21 +1309,31 @@ void decrease_indent(GtkTextBuffer *buffer, gint start_line, gint end_line)
 
 		if (old_tag != NULL) {
 			gint depth = tag_get_depth(old_tag);
-			if (depth <= 1) {
+
+			if (depth < 1) {
+				/* I think that should never happen */
+				g_assert_not_reached();
 				return;
 			}
-			depth--;
-			new_tag = buffer_get_depth_tag(buffer, depth);
 
 			gtk_text_buffer_get_iter_at_line(buffer, &end_iter, i);
 			gtk_text_iter_set_line_offset(&end_iter, 2);
 
+			if (depth == 1) {
+				g_printerr("Remove bullets\n");
+				remove_bullets(buffer, &start_iter, &end_iter);
+				return;
+			}
+
 			/* Remove old tag */
 			gtk_text_buffer_remove_tag(buffer, old_tag, &start_iter, &end_iter);
 
 			/* Delete old bullet */
 			gtk_text_buffer_delete(buffer, &start_iter, &end_iter);
 
+			depth--;
+			new_tag = buffer_get_depth_tag(buffer, depth);
+
 			/* Add new bullet with new tag */
 			add_bullets(buffer, i, i, new_tag);
 		}
@@ -1157,7 +1344,8 @@ void
 on_inc_indent_button_clicked			   (GtkAction		*action,
 											gpointer		 user_data)
 {
-	GtkTextBuffer *buffer = GTK_TEXT_BUFFER(user_data);
+	UserInterface *ui = (UserInterface*) user_data;
+	GtkTextBuffer *buffer = ui->buffer;
 	GtkTextIter start_iter, end_iter;
 	gint start_line, end_line;
 
@@ -1172,6 +1360,11 @@ on_inc_indent_button_clicked			   (GtkAction		*action,
 		increase_indent(buffer, start_line, start_line);
 	}
 
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
+	update_active_tags(buffer, &iter);
+	update_button_states(ui);
+
 	gtk_text_buffer_set_modified(buffer, TRUE);
 }
 
@@ -1180,7 +1373,8 @@ void
 on_dec_indent_button_clicked			   (GtkAction		*action,
 											gpointer		 user_data)
 {
-	GtkTextBuffer *buffer = GTK_TEXT_BUFFER(user_data);
+	UserInterface *ui = (UserInterface*) user_data;
+	GtkTextBuffer *buffer = ui->buffer;
 	GtkTextIter start_iter, end_iter;
 	gint start_line, end_line;
 
@@ -1195,6 +1389,11 @@ on_dec_indent_button_clicked			   (GtkAction		*action,
 		decrease_indent(buffer, start_line, start_line);
 	}
 
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
+	update_active_tags(buffer, &iter);
+	update_button_states(ui);
+
 	gtk_text_buffer_set_modified(buffer, TRUE);
 }
 
@@ -1214,8 +1413,7 @@ on_style_button_clicked                (GtkAction       *action,
 
 void on_find_button_clicked(GtkAction *action, gpointer user_data)
 {
-	Note *note = (Note*)user_data;
-	UserInterface *ui = note->ui;
+	UserInterface *ui = (UserInterface*)user_data;
 
 	if (ui->find_bar_is_visible) {
 		gtk_widget_hide_all(GTK_WIDGET(ui->find_bar));
@@ -1227,10 +1425,10 @@ void on_find_button_clicked(GtkAction *action, gpointer user_data)
 	}
 }
 
-void on_find_bar_search(GtkWidget *widget, Note *note)
+void on_find_bar_search(GtkWidget *widget, UserInterface *ui)
 {
-	GtkTextBuffer *buffer = note->ui->buffer;
-	GtkTextView *view = note->ui->view;
+	GtkTextBuffer *buffer = ui->buffer;
+	GtkTextView *view = ui->view;
 	gchar *search_str;
 	GtkTextMark *mark;
 	GtkTextIter iter, match_start, match_end;
@@ -1265,3 +1463,40 @@ void on_find_bar_close(GtkWidget *widget, UserInterface *ui)
 	gtk_widget_hide_all(widget);
 	ui->find_bar_is_visible = FALSE;
 }
+
+void
+on_back_button_clicked (GtkAction *action, gpointer user_data)
+{
+	AppData *app_data = app_data_get();
+
+	g_return_if_fail(app_data->current_element != NULL);
+	g_return_if_fail(app_data->current_element->prev != NULL);
+
+	app_data->current_element = app_data->current_element->prev;
+
+	ConboyNote *note = CONBOY_NOTE(app_data->current_element->data);
+	note_show(note, FALSE, TRUE, FALSE);
+}
+
+void
+on_forward_button_clicked (GtkAction *action, gpointer user_data)
+{
+	AppData *app_data = app_data_get();
+
+	g_return_if_fail(app_data->current_element != NULL);
+	g_return_if_fail(app_data->current_element->next != NULL);
+
+	app_data->current_element = app_data->current_element->next;
+
+	ConboyNote *note = CONBOY_NOTE(app_data->current_element->data);
+	note_show(note, FALSE, TRUE, FALSE);
+}
+
+void
+on_fullscreen_button_clicked		   (GtkAction		*action,
+										gpointer		 user_data)
+{
+	UserInterface *ui = (UserInterface*) user_data;
+	ui_helper_toggle_fullscreen(GTK_WINDOW(ui->window));
+}
+
